package job_info

/*
#cgo LDFLAGS: -lslurm
#include<stdlib.h>
#include<slurm/slurm.h>

inline uint8_t uint8_ptr(uint8_t* pointer) {
    if (NULL == pointer) {
    return -1;}
    return *pointer;
}
inline int8_t int8_ptr(int8_t* pointer) {
    if (NULL == pointer) {
    return -1;}
    return *pointer;
}
inline uint16_t uint16_ptr(uint16_t* pointer) {
    if (NULL == pointer) {
    return -1;}
    return *pointer;
}
inline int16_t int16_ptr(int16_t* pointer) {
    if (NULL == pointer) {
    return -1;}
    return *pointer;
}
inline uint32_t uint32_ptr(uint32_t* pointer) {
    if (NULL == pointer) {
    return -1;}
    return *pointer;
}
inline int32_t int32_ptr(int32_t* pointer) {
    if (NULL == pointer) {
    return -1;}
    return *pointer;
}
inline uint64_t uint64_ptr(uint64_t* pointer) {
    if (NULL == pointer) {
    return -1;}
    return *pointer;
}
inline int64_t int64_ptr(int16_t* pointer) {
    if (NULL == pointer) {
    return -1;}
    return *pointer;
}
 struct job_info_msg *get_job_info(){
   	struct job_info_msg* job_buffer;
      if(slurm_load_jobs ((time_t) NULL,
		&job_buffer, SHOW_ALL)) {
			return NULL;
		}
	return job_buffer;
 }
  struct job_info_msg *get_single_job_info(uint32_t id){
   	struct job_info_msg* job_buffer;
	if( slurm_load_job (&job_buffer, id, SHOW_DETAIL)) {
		return NULL;
	}
	return job_buffer;
 }

 struct job_info* job_from_list(struct job_info_msg *list, int i){
	 return &list->job_array[i];
}
 void free_job_buffer(void* buffer){

	slurm_free_job_info_msg ((struct job_info_msg*)buffer);
}

struct job_info_msg *get_user_job_info(uint32_t id){

	struct job_info_msg* job_buffer;
     	if( slurm_load_job_user(&job_buffer, id, SHOW_DETAIL))
     		return NULL;
 	return job_buffer;

}

int64_t get_job_endtime(int32_t jobid){
	time_t end_time;

	if(slurm_get_end_time (jobid, &end_time))
		return -1;
        else
		return end_time;


}

int char_len(char* c) {
	uint i = 0;
	while(c[i]!='\0') i++;
	return i;
 }


*/
import "C"

import "fmt"
import "os/user"
import "strconv"
import "time"

func Reason_to_string(state uint16) string {
	switch s := C.uint16_t(state); s {
	case C.WAIT_NO_REASON:
		return "wait no reason"
	}
	return "unkown reason"

}
func state_to_string(state uint32) string {
	switch s := C.uint32_t(state); s {
	case C.JOB_PENDING:
		return "Pending" /* queued waiting for initiation */
	case C.JOB_RUNNING:
		return "Running" /* allocated resources and executing */
	case C.JOB_SUSPENDED:
		return "Suspended" /* allocated resources, execution suspended */
	case C.JOB_COMPLETE:
		return "Complete" /* completed execution successfully */
	case C.JOB_CANCELLED:
		return "Cancelled" /* cancelled by user */
	case C.JOB_FAILED:
		return "Failed" /* completed execution unsuccessfully */
	case C.JOB_TIMEOUT:
		return "Timeout" /* terminated on reaching time limit */
	case C.JOB_NODE_FAIL:
		return "Node Fail" /* terminated on node failure */
	case C.JOB_PREEMPTED:
		return "Preempted" /* terminated due to preemption */
	}

	return "Unknow status" /* not a real state, last entry in table */
}

type Job_info struct {
	Account    string
	Alloc_node string
	Alloc_sid  uint32
	//array_bitmap void;
	Array_job_id     uint32
	Array_task_id    uint32
	Assoc_id         uint32
	Batch_flag       uint16
	Batch_host       string
	Boards_per_node  uint16
	Command          string
	Comment          string
	Contiguous       uint16
	Cores_per_socket uint16
	Cpus_per_task    uint16
	Dependency       string
	Derived_ec       uint32
	Eligible_time    int64
	End_time         int64
	Exc_nodes        string
	Exc_node_inx     int32
	Exit_code        uint32
	Features         string
	//gres_detail_str char**;
	Group_id          uint32
	Job_id            uint32
	Job_state         uint32
	Job_stateS        string
	Licenses          string
	Max_cpus          uint32
	Max_nodes         uint32
	Name              string
	Network           string
	Nodes             string
	Nice              uint32
	Node_inx          int32
	Ntasks_per_core   uint16
	Ntasks_per_node   uint16
	Ntasks_per_socket uint16
	Ntasks_per_board  uint16
	Num_cpus          uint32
	Num_nodes         uint32
	Partition         string
	Pn_min_memory     uint64
	Pn_min_cpus       uint16
	Pn_min_tmp_disk   uint32
	Preempt_time      int64
	Pre_sus_time      int64
	Priority          uint32
	Profile           uint32
	Qos               string
	Req_nodes         string
	Req_node_inx      int32
	Req_switch        uint32
	Requeue           uint16
	Resize_time       int64
	Restart_cnt       uint16
	Resv_name         string
	Shared            uint16
	Show_flags        uint16
	Sockets_per_board uint16
	Sockets_per_node  uint16
	Start_time        int64
	State_desc        string
	State_reason      uint16
	Submit_time       int64
	Suspend_time      int64
	Time_limit        uint32
	Time_min          uint32
	Threads_per_core  uint16
	User_id           uint32
	//accurate void;
	Wait4switch uint32
	Wckey       string
	Work_dir    string
}

func Job_info_convert_c_to_go(c_struct *C.struct_job_info) Job_info {
	var go_struct Job_info

	go_struct.Account = C.GoString(c_struct.account)
	go_struct.Alloc_node = C.GoString(c_struct.alloc_node)
	go_struct.Alloc_sid = uint32(c_struct.alloc_sid)
	go_struct.Array_job_id = uint32(c_struct.array_job_id)
	go_struct.Array_task_id = uint32(c_struct.array_task_id)
	go_struct.Assoc_id = uint32(c_struct.assoc_id)
	go_struct.Batch_flag = uint16(c_struct.batch_flag)
	go_struct.Batch_host = C.GoString(c_struct.batch_host)
	go_struct.Boards_per_node = uint16(c_struct.boards_per_node)
	go_struct.Command = C.GoString(c_struct.command)
	go_struct.Comment = C.GoString(c_struct.comment)
	go_struct.Contiguous = uint16(c_struct.contiguous)
	go_struct.Cores_per_socket = uint16(c_struct.cores_per_socket)
	go_struct.Cpus_per_task = uint16(c_struct.cpus_per_task)
	go_struct.Dependency = C.GoString(c_struct.dependency)
	go_struct.Derived_ec = uint32(c_struct.derived_ec)
	go_struct.Eligible_time = int64(c_struct.eligible_time)
	go_struct.End_time = int64(c_struct.end_time)
	go_struct.Exc_nodes = C.GoString(c_struct.exc_nodes)
	go_struct.Exc_node_inx = int32(C.int32_ptr(c_struct.exc_node_inx))
	go_struct.Exit_code = uint32(c_struct.exit_code)
	go_struct.Features = C.GoString(c_struct.features)
	go_struct.Group_id = uint32(c_struct.group_id)
	go_struct.Job_id = uint32(c_struct.job_id)
	go_struct.Job_state = uint32(c_struct.job_state)
	go_struct.Job_stateS = state_to_string(uint32(c_struct.job_state))
	go_struct.Licenses = C.GoString(c_struct.licenses)
	go_struct.Max_cpus = uint32(c_struct.max_cpus)
	go_struct.Max_nodes = uint32(c_struct.max_nodes)
	go_struct.Name = C.GoString(c_struct.name)
	go_struct.Network = C.GoString(c_struct.network)
	go_struct.Nodes = C.GoString(c_struct.nodes)
	go_struct.Nice = uint32(c_struct.nice)
	go_struct.Node_inx = int32(C.int32_ptr(c_struct.node_inx))
	go_struct.Ntasks_per_core = uint16(c_struct.ntasks_per_core)
	go_struct.Ntasks_per_node = uint16(c_struct.ntasks_per_node)
	go_struct.Ntasks_per_socket = uint16(c_struct.ntasks_per_socket)
	go_struct.Ntasks_per_board = uint16(c_struct.ntasks_per_board)
	go_struct.Num_cpus = uint32(c_struct.num_cpus)
	go_struct.Num_nodes = uint32(c_struct.num_nodes)
	go_struct.Partition = C.GoString(c_struct.partition)
	go_struct.Pn_min_memory = uint64(c_struct.pn_min_memory)
	go_struct.Pn_min_cpus = uint16(c_struct.pn_min_cpus)
	go_struct.Pn_min_tmp_disk = uint32(c_struct.pn_min_tmp_disk)
	go_struct.Preempt_time = int64(c_struct.preempt_time)
	go_struct.Pre_sus_time = int64(c_struct.pre_sus_time)
	go_struct.Priority = uint32(c_struct.priority)
	go_struct.Profile = uint32(c_struct.profile)
	go_struct.Qos = C.GoString(c_struct.qos)
	go_struct.Req_nodes = C.GoString(c_struct.req_nodes)
	go_struct.Req_node_inx = int32(C.int32_ptr(c_struct.req_node_inx))
	go_struct.Req_switch = uint32(c_struct.req_switch)
	go_struct.Requeue = uint16(c_struct.requeue)
	go_struct.Resize_time = int64(c_struct.resize_time)
	go_struct.Restart_cnt = uint16(c_struct.restart_cnt)
	go_struct.Resv_name = C.GoString(c_struct.resv_name)
	go_struct.Shared = uint16(c_struct.shared)
	go_struct.Show_flags = uint16(c_struct.show_flags)
	go_struct.Sockets_per_board = uint16(c_struct.sockets_per_board)
	go_struct.Sockets_per_node = uint16(c_struct.sockets_per_node)
	go_struct.Start_time = int64(c_struct.start_time)
	go_struct.State_desc = C.GoString(c_struct.state_desc)
	go_struct.State_reason = uint16(c_struct.state_reason)
	go_struct.Submit_time = int64(c_struct.submit_time)
	go_struct.Suspend_time = int64(c_struct.suspend_time)
	go_struct.Time_limit = uint32(c_struct.time_limit)
	go_struct.Time_min = uint32(c_struct.time_min)
	go_struct.Threads_per_core = uint16(c_struct.threads_per_core)
	go_struct.User_id = uint32(c_struct.user_id)
	go_struct.Wait4switch = uint32(c_struct.wait4switch)
	go_struct.Wckey = C.GoString(c_struct.wckey)
	go_struct.Work_dir = C.GoString(c_struct.work_dir)
	return go_struct
}

func Print_Job_info(go_struct Job_info) {
	fmt.Printf("%s:\t %s\n", "account", go_struct.Account)
	fmt.Printf("%s:\t %s\n", "alloc node", go_struct.Alloc_node)
	fmt.Printf("%s:\t %d\n", "alloc sid", go_struct.Alloc_sid)
	fmt.Printf("%s:\t %d\n", "array job id", go_struct.Array_job_id)
	fmt.Printf("%s:\t %d\n", "array task id", go_struct.Array_task_id)
	fmt.Printf("%s:\t %d\n", "assoc id", go_struct.Assoc_id)
	fmt.Printf("%s:\t %d\n", "batch flag", go_struct.Batch_flag)
	fmt.Printf("%s:\t %s\n", "batch host", go_struct.Batch_host)
	fmt.Printf("%s:\t %d\n", "boards per node", go_struct.Boards_per_node)
	fmt.Printf("%s:\t %s\n", "command", go_struct.Command)
	fmt.Printf("%s:\t %s\n", "comment", go_struct.Comment)
	fmt.Printf("%s:\t %d\n", "contiguous", go_struct.Contiguous)
	fmt.Printf("%s:\t %d\n", "cores per socket", go_struct.Cores_per_socket)
	fmt.Printf("%s:\t %d\n", "cpus per task", go_struct.Cpus_per_task)
	fmt.Printf("%s:\t %s\n", "dependency", go_struct.Dependency)
	fmt.Printf("%s:\t %d\n", "derived ec", go_struct.Derived_ec)
	fmt.Printf("%s:\t %d\n", "eligible time", go_struct.Eligible_time)
	fmt.Printf("%s:\t %d\n", "end time", go_struct.End_time)
	fmt.Printf("%s:\t %s\n", "exc nodes", go_struct.Exc_nodes)
	fmt.Printf("%s:\t %d\n", "exc node inx", go_struct.Exc_node_inx)
	fmt.Printf("%s:\t %d\n", "exit code", go_struct.Exit_code)
	fmt.Printf("%s:\t %s\n", "features", go_struct.Features)
	fmt.Printf("%s:\t %d\n", "group id", go_struct.Group_id)
	fmt.Printf("%s:\t %d\n", "job id", go_struct.Job_id)
	fmt.Printf("%s:\t %d\n", "job state", go_struct.Job_state)
	fmt.Printf("%s:\t %s\n", "licenses", go_struct.Licenses)
	fmt.Printf("%s:\t %d\n", "max cpus", go_struct.Max_cpus)
	fmt.Printf("%s:\t %d\n", "max nodes", go_struct.Max_nodes)
	fmt.Printf("%s:\t %s\n", "name", go_struct.Name)
	fmt.Printf("%s:\t %s\n", "network", go_struct.Network)
	fmt.Printf("%s:\t %s\n", "nodes", go_struct.Nodes)
	fmt.Printf("%s:\t %d\n", "nice", go_struct.Nice)
	fmt.Printf("%s:\t %d\n", "node inx", go_struct.Node_inx)
	fmt.Printf("%s:\t %d\n", "ntasks per core", go_struct.Ntasks_per_core)
	fmt.Printf("%s:\t %d\n", "ntasks per node", go_struct.Ntasks_per_node)
	fmt.Printf("%s:\t %d\n", "ntasks per socket", go_struct.Ntasks_per_socket)
	fmt.Printf("%s:\t %d\n", "ntasks per board", go_struct.Ntasks_per_board)
	fmt.Printf("%s:\t %d\n", "num cpus", go_struct.Num_cpus)
	fmt.Printf("%s:\t %d\n", "num nodes", go_struct.Num_nodes)
	fmt.Printf("%s:\t %s\n", "partition", go_struct.Partition)
	fmt.Printf("%s:\t %d\n", "pn min memory", go_struct.Pn_min_memory)
	fmt.Printf("%s:\t %d\n", "pn min cpus", go_struct.Pn_min_cpus)
	fmt.Printf("%s:\t %d\n", "pn min tmp disk", go_struct.Pn_min_tmp_disk)
	fmt.Printf("%s:\t %d\n", "preempt time", go_struct.Preempt_time)
	fmt.Printf("%s:\t %d\n", "pre sus time", go_struct.Pre_sus_time)
	fmt.Printf("%s:\t %d\n", "priority", go_struct.Priority)
	fmt.Printf("%s:\t %d\n", "profile", go_struct.Profile)
	fmt.Printf("%s:\t %s\n", "qos", go_struct.Qos)
	fmt.Printf("%s:\t %s\n", "req nodes", go_struct.Req_nodes)
	fmt.Printf("%s:\t %d\n", "req node inx", go_struct.Req_node_inx)
	fmt.Printf("%s:\t %d\n", "req switch", go_struct.Req_switch)
	fmt.Printf("%s:\t %d\n", "requeue", go_struct.Requeue)
	fmt.Printf("%s:\t %d\n", "resize time", go_struct.Resize_time)
	fmt.Printf("%s:\t %d\n", "restart cnt", go_struct.Restart_cnt)
	fmt.Printf("%s:\t %s\n", "resv name", go_struct.Resv_name)
	fmt.Printf("%s:\t %d\n", "shared", go_struct.Shared)
	fmt.Printf("%s:\t %d\n", "show flags", go_struct.Show_flags)
	fmt.Printf("%s:\t %d\n", "sockets per board", go_struct.Sockets_per_board)
	fmt.Printf("%s:\t %d\n", "sockets per node", go_struct.Sockets_per_node)
	fmt.Printf("%s:\t %d\n", "start time", go_struct.Start_time)
	fmt.Printf("%s:\t %s\n", "state desc", go_struct.State_desc)
	fmt.Printf("%s:\t %d\n", "state reason", go_struct.State_reason)
	fmt.Printf("%s:\t %d\n", "submit time", go_struct.Submit_time)
	fmt.Printf("%s:\t %d\n", "suspend time", go_struct.Suspend_time)
	fmt.Printf("%s:\t %d\n", "time limit", go_struct.Time_limit)
	fmt.Printf("%s:\t %d\n", "time min", go_struct.Time_min)
	fmt.Printf("%s:\t %d\n", "threads per core", go_struct.Threads_per_core)
	fmt.Printf("%s:\t %d\n", "user id", go_struct.User_id)
	fmt.Printf("%s:\t %d\n", "wait4switch", go_struct.Wait4switch)
	fmt.Printf("%s:\t %s\n", "wckey", go_struct.Wckey)
	fmt.Printf("%s:\t %s\n", "work dir", go_struct.Work_dir)
}

type Job_info_msg struct {
	Last_update  int64
	Record_count uint32
	Error_code   uint32
	Job_list     []Job_info
}

func Get_job_runtime(job Job_info) time.Duration {

	start_time := time.Unix(job.Start_time, 0)
	current_time := time.Now()
	diff := current_time.Sub(start_time).Round(time.Second)
	if int64(diff) < 0 {
		return 0
	}
	end_time := time.Unix(job.End_time, 0)
	diff2 := current_time.Sub(end_time)
	if int64(diff2) < 0 {
		return diff
	}
	return end_time.Sub(start_time)
}

func Get_job_endtime(id uint32) time.Time {
	c_time := C.get_job_endtime(C.int32_t(id))
	return time.Unix(int64(c_time), 0)

}

func Get_all_jobs() Job_info_msg {
	var go_job_buffer Job_info_msg
	c_job_buffer := C.get_job_info()
	if c_job_buffer == nil {
		go_job_buffer.Last_update = int64(0)
		go_job_buffer.Record_count = uint32(0)
		go_job_buffer.Error_code = uint32(C.slurm_get_errno())
		return go_job_buffer
	}

	go_job_buffer.Last_update = int64(c_job_buffer.last_update)
	go_job_buffer.Record_count = uint32(c_job_buffer.record_count)
	go_job_buffer.Job_list = make([]Job_info, c_job_buffer.record_count, c_job_buffer.record_count)
	for i := uint32(0); i < go_job_buffer.Record_count; i++ {
		job := C.job_from_list(c_job_buffer, C.int(i))
		go_job := Job_info_convert_c_to_go(job)
		go_job_buffer.Job_list[i] = go_job
	}
	C.slurm_free_job_info_msg(c_job_buffer)

	return go_job_buffer
}

func Get_job(id uint32) Job_info_msg {
	var go_job_buffer Job_info_msg
	c_job_buffer := C.get_single_job_info(C.uint32_t(id))
	if c_job_buffer == nil {
		go_job_buffer.Last_update = int64(0)
		go_job_buffer.Record_count = uint32(0)
		go_job_buffer.Error_code = uint32(C.slurm_get_errno())
		return go_job_buffer
	}
	go_job_buffer.Last_update = int64(c_job_buffer.last_update)
	go_job_buffer.Record_count = uint32(c_job_buffer.record_count)
	go_job_buffer.Job_list = make([]Job_info, c_job_buffer.record_count, c_job_buffer.record_count)
	for i := uint32(0); i < go_job_buffer.Record_count; i++ {
		job := C.job_from_list(c_job_buffer, C.int(i))
		go_job := Job_info_convert_c_to_go(job)
		go_job_buffer.Job_list[i] = go_job
	}
	C.slurm_free_job_info_msg(c_job_buffer)

	return go_job_buffer
}

func Get_user_jobs(name string) Job_info_msg {

	var go_job_buffer Job_info_msg
	user, err := user.Lookup(name)

	if err != nil {
		fmt.Printf("Error %s\n", err.Error())
		go_job_buffer.Last_update = int64(0)
		go_job_buffer.Record_count = uint32(0)
		go_job_buffer.Error_code = C.ESLURMD_UID_NOT_FOUND

		return go_job_buffer

	}

	userid, _ := strconv.Atoi(user.Uid)
	c_job_buffer := C.get_user_job_info(C.uint32_t(userid))

	if c_job_buffer == nil {
		go_job_buffer.Last_update = int64(0)
		go_job_buffer.Record_count = uint32(0)
		go_job_buffer.Error_code = uint32(C.slurm_get_errno())
		return go_job_buffer
	}
	go_job_buffer.Last_update = int64(c_job_buffer.last_update)
	go_job_buffer.Record_count = uint32(c_job_buffer.record_count)
	go_job_buffer.Job_list = make([]Job_info, c_job_buffer.record_count, c_job_buffer.record_count)
	for i := uint32(0); i < go_job_buffer.Record_count; i++ {
		job := C.job_from_list(c_job_buffer, C.int(i))
		go_job := Job_info_convert_c_to_go(job)
		go_job_buffer.Job_list[i] = go_job
	}
	C.slurm_free_job_info_msg(c_job_buffer)

	return go_job_buffer

}
