package slurmer

import (
	nodeinfo "code.gitlink.org.cn/JCCE/PCM.git/adaptor/pcm_slurm/cgo_tianhe/src/slurm/nodeinfo"
	pbslurm "code.gitlink.org.cn/JCCE/PCM.git/adaptor/pcm_slurm/gen/idl"
	ssh "code.gitlink.org.cn/JCCE/PCM.git/common/ssh"
	"context"
)

type SlurmStruct struct {
	version string
}

func SelectTianhe() (Slurmer, error) {

	return &SlurmStruct{
		version: "2.6.9.1",
	}, nil
}

func (slurmStruct SlurmStruct) AddQos(ctx context.Context, req *pbslurm.AddQosReq) (*pbslurm.AddQosResp, error) {
	cmd := "/usr/local/bin/sacctmgr add qos "
	cmd = cmd + req.SlurmdbQosRec.Name

	if len(req.SlurmdbQosRec.Description) != 0 {
		cmd = cmd + " Description=" + req.SlurmdbQosRec.Description
	}

	if len(req.SlurmdbQosRec.Name) != 0 {
		cmd = cmd + " Name=" + req.SlurmdbQosRec.Name
	}

	cmd = cmd + " -i"

	result := ssh.ExecCommand(cmd)
	resp := pbslurm.AddQosResp{}

	resp.Result = result
	return &resp, nil
}

func (slurmStruct SlurmStruct) GetQosByName(ctx context.Context, req *pbslurm.QosReq) (*pbslurm.QosResp, error) {
	qos := GetQosByName(req.Name)
	var resp = pbslurm.QosResp{}
	slurmdbRosRec := []*pbslurm.SlurmdbQosRec{}
	slurmdbRosRec = append(slurmdbRosRec, &qos)
	resp.SlurmdbQosRec = slurmdbRosRec

	return &resp, nil
}

func (slurmStruct SlurmStruct) GetSlurmdbJobs(ctx context.Context, req *pbslurm.SlurmDbJobReq) (*pbslurm.SlurmDbJobResp, error) {
	slurmdbJobs := GetAllSlurmdbJobs()
	return getSlurmdbJobResp(slurmdbJobs), nil
}

func (slurmStruct SlurmStruct) GetSlurmdbJobById(ctx context.Context, req *pbslurm.SlurmDbJobReq) (*pbslurm.SlurmDbJobResp, error) {
	slurmdbjob := GetSlurmdbJobById(int(req.JobId))
	return getSlurmdbJobResp(slurmdbjob), nil
}

func (slurmStruct SlurmStruct) DeleteQos(ctx context.Context, req *pbslurm.DeleteQosReq) (*pbslurm.DeleteQosResp, error) {

	cmd := "/usr/local/bin/sacctmgr delete qos "
	cmd = cmd + req.Names
	cmd = cmd + " -i"

	result := ssh.ExecCommand(cmd)

	resp := pbslurm.DeleteQosResp{}
	resp.Result = result

	return &resp, nil
}

func (slurmStruct SlurmStruct) GetQos(ctx context.Context, req *pbslurm.QosReq) (*pbslurm.QosResp, error) {
	qoslist := Get_Qos()
	var resp = pbslurm.QosResp{}
	resp.SlurmdbQosRec = qoslist
	return &resp, nil
}

func (slurmStruct SlurmStruct) GetDiag(ctx context.Context, req *pbslurm.DiagReq) (*pbslurm.DiagResp, error) {
	stats := Get_diag()

	var resp = pbslurm.DiagResp{}
	statsInfoResponseMsg := pbslurm.StatsInfoResponseMsg{}
	statsInfoResponseMsg.AgentQueueSize = stats.Agent_queue_size
	statsInfoResponseMsg.BfBackfilledJobs = stats.Bf_backfilled_jobs
	statsInfoResponseMsg.BfActive = stats.Bf_active
	statsInfoResponseMsg.BfCycleCounter = stats.Bf_cycle_counter
	statsInfoResponseMsg.BfCycleLast = stats.Bf_cycle_last
	statsInfoResponseMsg.BfCycleMax = stats.Bf_cycle_max
	statsInfoResponseMsg.BfCycleSum = stats.Bf_cycle_sum
	statsInfoResponseMsg.BfDepthSum = stats.Bf_depth_sum
	statsInfoResponseMsg.BfDepthTrySum = stats.Bf_depth_try_sum
	statsInfoResponseMsg.BfLastBackfilledJobs = stats.Bf_last_backfilled_jobs
	statsInfoResponseMsg.BfLastDepth = stats.Bf_last_depth
	statsInfoResponseMsg.BfLastDepthTry = stats.Bf_last_depth_try
	statsInfoResponseMsg.BfQueueLen = stats.Bf_queue_len
	statsInfoResponseMsg.BfQueueLenSum = stats.Bf_queue_len_sum
	statsInfoResponseMsg.BfWhenLastCycle = stats.Bf_when_last_cycle
	statsInfoResponseMsg.JobsFailed = stats.Jobs_failed
	statsInfoResponseMsg.JobsCanceled = stats.Jobs_canceled
	statsInfoResponseMsg.JobsCompleted = stats.Jobs_completed
	statsInfoResponseMsg.JobsStarted = stats.Jobs_started
	statsInfoResponseMsg.JobsSubmitted = stats.Jobs_submitted
	statsInfoResponseMsg.PartsPacked = stats.Parts_packed
	statsInfoResponseMsg.ReqTimeStart = stats.Req_time_start
	statsInfoResponseMsg.ReqTime = stats.Req_time
	statsInfoResponseMsg.ScheduleCycleCounter = stats.Schedule_cycle_counter
	statsInfoResponseMsg.ScheduleCycleDepth = stats.Schedule_cycle_depth
	statsInfoResponseMsg.ScheduleCycleLast = stats.Schedule_cycle_last
	statsInfoResponseMsg.ScheduleCycleMax = stats.Schedule_cycle_max
	statsInfoResponseMsg.ScheduleCycleSum = stats.Schedule_cycle_sum
	statsInfoResponseMsg.ScheduleQueueLen = stats.Schedule_queue_len
	statsInfoResponseMsg.ServerThreadCount = stats.Server_thread_count

	resp.StatsInfoResponseMsg = &statsInfoResponseMsg

	return &resp, nil
}

func (slurmStruct SlurmStruct) ListNodes(ctx context.Context, req *pbslurm.ListNodesReq) (*pbslurm.ListNodesResp, error) {
	nodeList := nodeinfo.Get_all_nodes()

	var resp = pbslurm.ListNodesResp{}
	for _, node := range nodeList.Node_list {
		nodeInfoResult := pbslurm.NodeInfo{}
		nodeInfoResult.Cpus = int32(node.Cpus)
		nodeInfoResult.Boards = int32(node.Boards)
		nodeInfoResult.RealMemory = int32(node.Real_memory)
		nodeInfoResult.Sockets = int32(node.Sockets)
		nodeInfoResult.Threads = int32(node.Threads)
		resp.NodeInfos = append(resp.NodeInfos, &nodeInfoResult)
	}

	return &resp, nil
}

func (slurmStruct SlurmStruct) GetAllJobs(ctx context.Context, req *pbslurm.JobInfoMsgReq) (*pbslurm.JobInfoMsgResp, error) {
	job_info_msg := Get_all_jobs()
	return getJobInfoMsgResp(job_info_msg), nil
}

func (slurmStruct SlurmStruct) GetJob(ctx context.Context, req *pbslurm.JobInfoMsgReq) (*pbslurm.JobInfoMsgResp, error) {
	job_info_msg := Get_job(req.JobId)
	return getJobInfoMsgResp(job_info_msg), nil
}

func (slurmStruct SlurmStruct) SubmitJob(ctx context.Context, req *pbslurm.SubmitJobReq) (*pbslurm.SubmitJobResp, error) {
	var job Job_descriptor
	convert_jobDescriptor_pb_to_go(&job, req.Data)

	submit_response_msg := Submit_job(&job)
	var resp = pbslurm.SubmitJobResp{}
	submitResponseMsg := pbslurm.SubmitResponseMsg{}
	submitResponseMsg.JobId = submit_response_msg.Job_id
	submitResponseMsg.StepId = submit_response_msg.Step_id
	submitResponseMsg.ErrorCode = submit_response_msg.Error_code

	resp.SubmitResponseMsg = append(resp.SubmitResponseMsg, &submitResponseMsg)

	return &resp, nil
}

func (slurmStruct SlurmStruct) UpdateJob(ctx context.Context, req *pbslurm.UpdateJobReq) (*pbslurm.UpdateJobResp, error) {
	var options Update_job_options
	convert_update_job_options_pb_to_go(&options, req.Data)

	errorCode := Update_job(options, req.JobId)
	var resp = pbslurm.UpdateJobResp{}
	resp.ErrorCode = errorCode
	return &resp, nil
}

func (slurmStruct SlurmStruct) DeleteJob(ctx context.Context, req *pbslurm.DeleteJobReq) (*pbslurm.DeleteJobResp, error) {
	errorCode := Delete_job(req.JobId)
	var resp = pbslurm.DeleteJobResp{}
	resp.ErrorCode = errorCode
	return &resp, nil
}

func getJobInfoMsgResp(job_info_msg Job_info_msg) *pbslurm.JobInfoMsgResp {
	var resp = pbslurm.JobInfoMsgResp{}
	var jobInfoList = []*pbslurm.JobInfo{}

	for _, e := range job_info_msg.Job_list {
		jobInfo := pbslurm.JobInfo{}
		jobInfo.ArrayJobId = e.Array_job_id
		jobInfo.Account = e.Account
		jobInfo.AllocNode = e.Alloc_node
		jobInfo.AllocSid = e.Alloc_sid
		jobInfo.ArrayTaskId = uint32(e.Array_task_id)
		jobInfo.AssocId = e.Assoc_id
		jobInfo.BatchHost = e.Batch_host
		jobInfo.BatchFlag = uint32(e.Batch_flag)
		jobInfo.BatchScript = e.Batch_script
		jobInfo.BoardsPerNode = uint32(e.Boards_per_node)
		jobInfo.Comment = e.Comment
		jobInfo.Command = e.Command
		jobInfo.Contiguous = uint32(e.Contiguous)
		jobInfo.CoresPerSocket = uint32(e.Cores_per_socket)
		jobInfo.CpusPerTask = uint32(e.Cpus_per_task)
		jobInfo.DerivedEc = e.Derived_ec
		jobInfo.Dependency = e.Dependency
		jobInfo.EndTime = e.End_time
		jobInfo.ExcNodeInx = e.Exc_node_inx
		jobInfo.EligibleTime = e.Eligible_time
		jobInfo.ExcNodes = e.Exc_nodes
		jobInfo.ExitCode = e.Exit_code
		jobInfo.Features = e.Features
		jobInfo.Gres = e.Gres
		jobInfo.GroupId = e.Group_id
		jobInfo.JobId = e.Job_id
		jobInfo.JobState = uint32(e.Job_state)
		jobInfo.Licenses = e.Licenses
		jobInfo.MaxNodes = e.Max_nodes
		jobInfo.MaxCpus = e.Max_cpus
		jobInfo.Name = e.Name
		jobInfo.Nice = uint32(e.Nice)
		jobInfo.Network = e.Network
		jobInfo.Nodes = e.Nodes
		jobInfo.NodeInx = e.Node_inx
		jobInfo.NtasksPerBoard = uint32(e.Ntasks_per_board)
		jobInfo.NtasksPerCore = uint32(e.Ntasks_per_core)
		jobInfo.NtasksPerNode = uint32(e.Ntasks_per_node)
		jobInfo.NtasksPerSocket = uint32(e.Ntasks_per_socket)
		jobInfo.NumCpus = e.Num_cpus
		jobInfo.NumNodes = e.Num_nodes
		jobInfo.PnMinCpus = uint32(e.Pn_min_cpus)
		jobInfo.Partition = e.Partition
		jobInfo.PnMinMemory = e.Pn_min_memory
		jobInfo.PnMinTmpDisk = e.Pn_min_tmp_disk
		jobInfo.PreemptTime = e.Preempt_time
		jobInfo.PreSusTime = e.Pre_sus_time
		jobInfo.Priority = e.Priority
		jobInfo.Profile = e.Profile
		jobInfo.Qos = e.Qos
		jobInfo.Requeue = uint32(e.Requeue)
		jobInfo.ReqNodes = e.Req_nodes
		jobInfo.ReqNodeInx = e.Req_node_inx
		jobInfo.ReqSwitch = e.Req_switch
		jobInfo.ResizeTime = e.Resize_time
		jobInfo.RestartCnt = uint32(e.Restart_cnt)
		jobInfo.ResvName = e.Resv_name
		jobInfo.ShowFlags = uint32(e.Show_flags)
		jobInfo.Shared = uint32(e.Shared)
		jobInfo.SocketsPerBoard = uint32(e.Sockets_per_board)
		jobInfo.StartTime = e.Start_time
		jobInfo.SocketsPerNode = uint32(e.Sockets_per_node)
		jobInfo.StateDesc = e.State_desc
		jobInfo.SubmitTime = e.Submit_time
		jobInfo.SuspendTime = e.Suspend_time
		jobInfo.StateReason = uint32(e.State_reason)
		jobInfo.TimeMin = e.Time_min
		jobInfo.TimeLimit = e.Time_limit
		jobInfo.ThreadsPerCore = uint32(e.Threads_per_core)
		jobInfo.UserId = e.User_id
		jobInfo.Wckey = e.Wckey
		jobInfo.WorkDir = e.Work_dir
		jobInfo.Wait4Switch = e.Wait4switch
		jobInfoList = append(jobInfoList, &jobInfo)
	}

	jobInfoMsg := pbslurm.JobInfoMsg{}
	jobInfoMsg.RecordCount = job_info_msg.Record_count
	jobInfoMsg.LastUpdate = job_info_msg.Last_update
	jobInfoMsg.JobList = jobInfoList

	resp.JobInfoMsg = append(resp.JobInfoMsg, &jobInfoMsg)

	return &resp
}

func convert_jobDescriptor_pb_to_go(job *Job_descriptor, data *pbslurm.JobDescriptor) {
	job.Account = data.Account
	job.Argc = data.Argc
	job.Alloc_node = data.AllocNode
	job.Alloc_sid = data.AllocSid
	job.Acctg_freq = data.AcctgFreq
	job.Alloc_resp_port = uint16(data.AllocRespPort)
	job.Array_inx = data.ArrayInx
	if data.Argv != nil {
		for _, v := range data.Argv {
			job.Argv = append(job.Argv, v.Argv)
		}
	}
	job.Begin_time = data.BeginTime
	job.Boards_per_node = uint16(data.BoardsPerNode)
	job.Comment = data.Comment
	job.Contiguous = uint16(data.Contiguous)
	job.Ckpt_dir = data.CkptDir
	job.Ckpt_interval = uint16(data.CkptInterval)
	job.Cpu_bind = data.CpuBind
	job.Cores_per_socket = uint16(data.CoresPerSocket)
	job.Cpu_bind_type = uint16(data.CpuBindType)
	job.Cpus_per_task = uint16(data.CpusPerTask)
	job.Dependency = data.Dependency
	if data.Environment != nil {
		for _, v := range data.Environment {
			job.Environment = append(job.Environment, v.Environment)
		}
	}
	job.End_time = data.EndTime
	job.Exc_nodes = data.ExcNodes
	job.Env_size = data.EnvSize
	job.Features = data.Features
	job.Gres = data.Gres
	job.Group_id = data.GroupId
	job.Immediate = uint16(data.Immediate)
	job.Job_id = data.JobId
	job.Kill_on_node_fail = uint16(data.KillOnNodeFail)
	job.Licenses = data.Licenses
	job.Mail_type = uint16(data.MailType)
	job.Max_cpus = data.MaxCpus
	job.Max_nodes = data.MaxNodes
	job.Mail_user = data.MailUser
	job.Mem_bind = data.MemBind
	job.Mem_bind_type = uint16(data.MemBindType)
	job.Min_cpus = data.MinCpus
	job.Min_nodes = data.MinNodes
	job.Network = data.Network
	job.Ntasks_per_node = uint16(data.NtasksPerNode)
	job.Ntasks_per_socket = uint16(data.NtasksPerSocket)
	job.Nice = uint16(data.Nice)
	job.Ntasks_per_core = uint16(data.NtasksPerCore)
	job.Ntasks_per_board = uint16(data.NtasksPerBoard)
	job.Name = data.Name
	job.Num_tasks = data.NumTasks
	job.Overcommit = uint8(data.Overcommit)
	job.Open_mode = uint8(data.OpenMode)
	job.Other_port = uint16(data.OtherPort)
	job.Profile = data.Profile
	job.Priority = data.Priority
	job.Pn_min_tmp_disk = data.PnMinTmpDisk
	job.Partition = data.Partition
	job.Pn_min_memory = data.PnMinMemory
	job.Pn_min_cpus = uint16(data.PnMinCpus)
	job.Plane_size = uint16(data.PlaneSize)
	job.Qos = data.Qos
	job.Req_nodes = data.ReqNodes
	job.Requeue = uint16(data.Requeue)
	job.Req_switch = data.ReqSwitch
	job.Reboot = uint16(data.Reboot)
	job.Reservation = data.Reservation
	job.Resp_host = data.RespHost
	job.Rotate = uint16(data.Rotate)
	job.Shared = uint16(data.Shared)
	job.Script = data.Script
	job.Sockets_per_node = uint16(data.SocketsPerNode)
	job.Spank_job_env_size = data.SpankJobEnvSize
	job.Sockets_per_board = uint16(data.SocketsPerBoard)
	job.Std_err = data.StdErr
	job.Std_in = data.StdIn
	job.Std_out = data.StdOut
	job.Time_min = data.TimeMin
	job.Time_limit = data.TimeLimit
	job.Threads_per_core = uint16(data.ThreadsPerCore)
	job.Task_dist = uint16(data.TaskDist)
	job.User_id = data.UserId
	job.Work_dir = data.WorkDir
	job.Wckey = data.Wckey
	job.Wait4switch = data.Wait4Switch
	job.Wait_all_nodes = uint16(data.WaitAllNodes)
	job.Warn_signal = uint16(data.WarnSignal)
	job.Warn_time = uint16(data.WarnTime)
}

func convert_update_job_options_pb_to_go(options *Update_job_options, data *pbslurm.UpdateJobOptions) {
	options.Num_tasks = data.NumTasks
	options.Qos = data.Qos
	options.Partition = data.Partition
	options.Ntasks_per_core = uint16(data.NtasksPerCore)
	options.Max_nodes = data.MaxNodes
	options.Ntasks_per_socket = uint16(data.NtasksPerSocket)
	options.Ntasks_per_node = uint16(data.NtasksPerNode)
	options.Min_nodes = data.MinNodes
}

func getSlurmdbJobResp(slurmJobs []pbslurm.SlurmdbJobRec) *pbslurm.SlurmDbJobResp {
	var resp = pbslurm.SlurmDbJobResp{}
	var jobList = []*pbslurm.SlurmdbJobRec{}
	for _, e := range slurmJobs {
		job := pbslurm.SlurmdbJobRec{}
		job.Associd = e.Associd
		job.AllocCpu = e.AllocCpu
		job.AllocNodes = e.AllocNodes
		job.Account = e.Account
		job.Blockid = e.Blockid
		job.Cluster = e.Cluster
		job.DerivedEs = e.DerivedEs
		job.DerivedEc = e.DerivedEc
		job.Elapsed = e.Elapsed
		job.Eligible = e.Eligible
		job.End = e.End
		job.Exitcode = e.Exitcode
		job.Gid = e.Gid
		job.Jobname = e.Jobname
		job.Jobid = e.Jobid
		job.Lft = e.Lft
		job.Nodes = e.Nodes
		job.Partition = e.Partition
		job.Priority = e.Priority
		job.Qosid = e.Qosid
		job.ReqMem = e.ReqMem
		job.ReqCpus = e.ReqCpus
		job.Resvid = e.Resvid
		job.Requid = e.Requid
		job.ShowFull = e.ShowFull
		job.State = e.State
		job.Submit = e.Submit
		job.Requid = e.Requid
		job.Start = e.Start
		job.Suspended = e.Suspended
		job.SysCpuSec = e.SysCpuSec
		job.SysCpuUsec = e.SysCpuUsec
		job.TotCpuUsec = e.TotCpuUsec
		job.Timelimit = e.Timelimit
		job.TotCpuSec = e.TotCpuSec
		job.TrackSteps = e.TrackSteps
		job.User = e.User
		job.Uid = e.Uid
		job.UserCpuSec = e.UserCpuSec
		job.UserCpuUsec = e.UserCpuUsec
		job.Wckey = e.Wckey
		job.Wckeyid = e.Wckeyid
		jobList = append(jobList, &job)
	}
	resp.SlurmdbJobRec = jobList
	return &resp
}
