package slurmer

type Job_descriptor struct {
	Account    string /* charge to specified account */
	Acctg_freq string /* accounting polling intervals (seconds) */
	Alloc_node string /* node making resource allocation request
	 * NOTE: Normally set by slurm_submit* or
	 * slurm_allocate* function */
	Alloc_resp_port uint16 /* port to send allocation confirmation to */
	Alloc_sid       uint32 /* local sid making resource allocation request
	 * NOTE: Normally set by slurm_submit* or
	 * slurm_allocate* function
	 * NOTE: Also used for update flags, see
	 * ALLOC_SID_* flags */
	Argc      uint32   /* number of arguments to the script */
	Argv      []string /* arguments to the script */
	Array_inx string   /* job array index values */
	//void *array_bitmap;	/* NOTE: Set by slurmctld */
	Begin_time    int64  /* delay initiation until this time */
	Ckpt_interval uint16 /* periodically checkpoint this job */
	Ckpt_dir      string /* directory to store checkpoint images */
	Comment       string /* arbitrary comment (used by Moab scheduler) */
	Contiguous    uint16 /* 1 if job requires contiguous nodes,
	 * 0 otherwise,default=0 */
	Cpu_bind      string /* binding map for map/mask_cpu */
	Cpu_bind_type uint16 /* see cpu_bind_type_t */
	Dependency    string /* synchronize job execution with other jobs */
	End_time      int64  /* time by which job must complete, used for
	 * job update only now, possible deadline
	 * scheduling in the future */
	Environment []string /* environment variables to set for job,
	 *  name=value pairs, one per line */
	Env_size  uint32 /* element count in environment */
	Exc_nodes string /* comma separated list of nodes excluded
	 * from job's allocation, default NONE */
	Features string /* comma separated list of required features,
	 * default NONE */
	Gres string /* comma separated list of required generic
	 * resources, default NONE */
	Group_id  uint32 /* group to assume, if run as root. */
	Immediate uint16 /* 1 if allocate to run or fail immediately,
	 * 0 if to be queued awaiting resources */
	Job_id            uint32 /* job ID, default set by SLURM */
	Kill_on_node_fail uint16 /* 1 if node failure to kill job,
	 * 0 otherwise,default=1 */
	Licenses      string /* licenses required by the job */
	Mail_type     uint16 /* see MAIL_JOB_ definitions above */
	Mail_user     string /* user to receive notification */
	Mem_bind      string /* binding map for map/mask_cpu */
	Mem_bind_type uint16 /* see mem_bind_type_t */
	Name          string /* name of the job, default "" */
	Network       string /* network use spec */
	Nice          uint16 /* requested priority change,
	 * NICE_OFFSET == no change */
	Num_tasks uint32 /* number of tasks to be started,
	 * for batch only */
	Open_mode uint8 /* out/err open mode truncate or append,
	 * see OPEN_MODE_* */
	Other_port uint16 /* port to send various notification msg to */
	Overcommit uint8  /* over subscribe resources, for batch only */
	Partition  string /* name of requested partition,
	 * default in SLURM config */
	Plane_size uint16 /* plane size when task_dist =
	   SLURM_DIST_PLANE */
	Priority uint32 /* relative priority of the job,
	 * explicitly set only for user root,
	 * 0 == held (don't initiate) */
	Profile   uint32 /* Level of acct_gather_profile {all | none} */
	Qos       string /* Quality of Service */
	Resp_host string /* NOTE: Set by slurmctld */
	Req_nodes string /* comma separated list of required nodes
	 * default NONE */
	Requeue     uint16 /* enable or disable job requeue option */
	Reservation string /* name of reservation to use */
	Script      string /* the actual job script, default NONE */
	Shared      uint16 /* 1 if job can share nodes with other jobs,
	 * 0 if job needs exclusive access to the node,
	 * or NO_VAL to accept the system default.
	 * SHARED_FORCE to eliminate user control. */
	//char **spank_job_env;	 environment variables for job prolog/epilog
	// * scripts as set by SPANK plugins
	Spank_job_env_size uint32 /* element count in spank_env */
	Task_dist          uint16 /* see enum task_dist_state */
	Time_limit         uint32 /* maximum run time in minutes, default is
	 * partition limit */
	Time_min uint32 /* minimum run time in minutes, default is
	 * time_limit */
	User_id uint32 /* set only if different from current UID,
	 * can only be explicitly set by user root */
	Wait_all_nodes uint16 /* 0 to start job immediately after allocation
	 * 1 to start job after all nodes booted
	 * or NO_VAL to use system default */
	Warn_signal uint16 /* signal to send when approaching end time */
	Warn_time   uint16 /* time before end to send signal (seconds) */
	Work_dir    string /* pathname of working directory */

	/* job constraints: */
	Cpus_per_task uint16 /* number of processors required for
	 * each task */
	Min_cpus uint32 /* minimum number of processors required,
	 * default=0 */
	Max_cpus uint32 /* maximum number of processors required,
	 * default=0 */
	Min_nodes uint32 /* minimum number of nodes required by job,
	 * default=0 */
	Max_nodes uint32 /* maximum number of nodes usable by job,
	 * default=0 */
	Boards_per_node   uint16 /* boards per node required by job  */
	Sockets_per_board uint16 /* sockets per board required by job */
	Sockets_per_node  uint16 /* sockets per node required by job */
	Cores_per_socket  uint16 /* cores per socket required by job */
	Threads_per_core  uint16 /* threads per core required by job */
	Ntasks_per_node   uint16 /* number of tasks to invoke on each node */
	Ntasks_per_socket uint16 /* number of tasks to invoke on
	 * each socket */
	Ntasks_per_core  uint16 /* number of tasks to invoke on each core */
	Ntasks_per_board uint16 /* number of tasks to invoke on each board */
	Pn_min_cpus      uint16 /* minimum # CPUs per node, default=0 */
	Pn_min_memory    uint32 /* minimum real memory per node OR
	 * real memory per CPU | MEM_PER_CPU,
	 * default=0 (no limit) */
	Pn_min_tmp_disk uint32 /* minimum tmp disk per node,
	 * default=0 */

	/*
	 * The following parameters are only meaningful on a Blue Gene
	 * system at present. Some will be of value on other system. Don't remove these
	 * they are needed for LCRM and others that can't talk to the opaque data type
	 * select_jobinfo.
	 */
	//uint16_t geometry[HIGHEST_DIMENSIONS];	 node count in various
	// * dimensions, e.g. X, Y, and Z
	//uint16_t conn_type[HIGHEST_DIMENSIONS];	 see enum connection_type
	Reboot uint16 /* force node reboot before startup */
	Rotate uint16 /* permit geometry rotation if set */
	//char *blrtsimage;       /* BlrtsImage for block */
	//char *linuximage;       /* LinuxImage for block */
	//char *mloaderimage;     /* MloaderImage for block */
	//char *ramdiskimage;     /* RamDiskImage for block */

	/* End of Blue Gene specific values */
	Req_switch uint32 /* Minimum number of switches */
	//dynamic_plugin_data_t *select_jobinfo; /* opaque data type,
	// * SLURM internal use only */
	Std_err     string /* pathname of stderr */
	Std_in      string /* pathname of stdin */
	Std_out     string /* pathname of stdout */
	Wait4switch uint32 /* Maximum time to wait for minimum switches */
	Wckey       string /* wckey for job */
}

type Submit_response_msg struct {
	Job_id     uint32
	Step_id    uint32
	Error_code uint32
}

type Job_info struct {
	Account       string /* charge to specified account */
	Alloc_node    string /* local node making resource alloc */
	Alloc_sid     uint32 /* local sid making resource alloc */
	Array_job_id  uint32 /* job_id of a job array or 0 if N/A */
	Array_task_id uint16 /* task_id of a job array */
	Assoc_id      uint32 /* association id for job */
	Batch_flag    uint16 /* 1 if batch: queued job with script */
	Batch_host    string /* name of host running batch script */
	Batch_script  string /* contents of batch script */
	Command       string /* command to be executed, built from submitted
	 * job's argv and NULL for salloc command */
	Comment       string /* arbitrary comment (used by Moab scheduler) */
	Contiguous    uint16 /* 1 if job requires contiguous nodes */
	Cpus_per_task uint16 /* number of processors required for
	 * each task */
	Dependency    string /* synchronize job execution with other jobs */
	Derived_ec    uint32 /* highest exit code of all job steps */
	Eligible_time int64  /* time job is eligible for running */
	End_time      int64  /* time of termination, actual or expected */
	Exc_nodes     string /* comma separated list of excluded nodes */
	Exc_node_inx  int32  /* excluded list index pairs into node_table:
	 * start_range_1, end_range_1,
	 * start_range_2, .., -1  */
	Exit_code         uint32 /* exit code for job (status from wait call) */
	Features          string /* comma separated list of required features */
	Gres              string /* comma separated list of generic resources */
	Group_id          uint32 /* group job sumitted as */
	Job_id            uint32 /* job ID */
	Job_state         uint16 /* state of the job, see enum job_states */
	Licenses          string /* licenses required by the job */
	Max_cpus          uint32 /* maximum number of cpus usable by job */
	Max_nodes         uint32 /* maximum number of nodes usable by job */
	Boards_per_node   uint16 /* boards per node required by job   */
	Sockets_per_board uint16 /* sockets per board required by job */
	Sockets_per_node  uint16 /* sockets per node required by job  */
	Cores_per_socket  uint16 /* cores per socket required by job  */
	Threads_per_core  uint16 /* threads per core required by job  */
	Name              string /* name of the job */
	Network           string /* network specification */
	Nodes             string /* list of nodes allocated to job */
	Nice              uint16 /* requested priority change */
	Node_inx          int32  /* list index pairs into node_table for *nodes:
	 * start_range_1, end_range_1,
	 * start_range_2, .., -1  */
	Ntasks_per_core   uint16 /* number of tasks to invoke on each core */
	Ntasks_per_node   uint16 /* number of tasks to invoke on each node */
	Ntasks_per_socket uint16 /* number of tasks to invoke on each socket*/
	Ntasks_per_board  uint16 /* number of tasks to invoke on each board */

	Num_nodes       uint32 /* minimum number of nodes required by job */
	Num_cpus        uint32 /* minimum number of cpus required by job */
	Partition       string /* name of assigned partition */
	Pn_min_memory   uint32 /* minimum real memory per node, default=0 */
	Pn_min_cpus     uint16 /* minimum # CPUs per node, default=0 */
	Pn_min_tmp_disk uint32 /* minimum tmp disk per node, default=0 */
	Pre_sus_time    int64  /* time job ran prior to last suspend */
	Priority        uint32 /* relative priority of the job,
	 * 0=held, 1=required nodes DOWN/DRAINED */
	Profile      uint32 /* Level of acct_gather_profile {all | none} */
	Qos          string /* Quality of Service */
	Req_nodes    string /* comma separated list of required nodes */
	Req_node_inx int32  /* required list index pairs into node_table:
	 * start_range_1, end_range_1,
	 * start_range_2, .., -1  */
	Req_switch  uint32 /* Minimum number of switches */
	Requeue     uint16 /* enable or disable job requeue option */
	Resize_time int64  /* time of latest size change */
	Restart_cnt uint16 /* count of job restarts */
	Resv_name   string /* reservation name */
	/*dynamic_plugin_data_t *select_jobinfo;*/ /* opaque data type,
	* process using
	* slurm_get_select_jobinfo()
	 */
	/*job_resources_t *job_resrcs;*/ /* opaque data type, job resources */
	Shared                           uint16 /* 1 if job can share nodes with other jobs */
	Show_flags                       uint16 /* conveys level of details requested */
	Start_time                       int64  /* time execution begins, actual or expected */
	State_desc                       string /* optional details for state_reason */
	State_reason                     uint16 /* reason job still pending or failed, see
	 * slurm.h:enum job_state_reason */
	Submit_time  int64  /* time of job submission */
	Suspend_time int64  /* time job last suspended or resumed */
	Time_limit   uint32 /* maximum run time in minutes or INFINITE */
	Time_min     uint32 /* minimum run time in minutes or INFINITE */
	User_id      uint32 /* user the job runs as */
	Preempt_time int64  /* preemption signal time */
	Wait4switch  uint32 /* Maximum time to wait for minimum switches */
	Wckey        string /* wckey for job */
	Work_dir     string /* pathname of working directory */
}

type Job_info_msg struct {
	Last_update  int64
	Record_count uint32
	Job_list     []Job_info
}

type Update_job_options struct {
	Partition         string
	Qos               string
	Num_tasks         uint32
	Ntasks_per_node   uint16
	Ntasks_per_socket uint16
	Ntasks_per_core   uint16
	Min_nodes         uint32
	Max_nodes         uint32
}

type Stats_info_response_msg struct {
	Parts_packed        uint32
	Req_time            int64
	Req_time_start      int64
	Server_thread_count uint32
	Agent_queue_size    uint32

	Schedule_cycle_max     uint32
	Schedule_cycle_last    uint32
	Schedule_cycle_sum     uint32
	Schedule_cycle_counter uint32
	Schedule_cycle_depth   uint32
	Schedule_queue_len     uint32

	Jobs_submitted uint32
	Jobs_started   uint32
	Jobs_completed uint32
	Jobs_canceled  uint32
	Jobs_failed    uint32

	Bf_backfilled_jobs      uint32
	Bf_last_backfilled_jobs uint32
	Bf_cycle_counter        uint32
	Bf_cycle_sum            uint32
	Bf_cycle_last           uint32
	Bf_cycle_max            uint32
	Bf_last_depth           uint32
	Bf_last_depth_try       uint32
	Bf_depth_sum            uint32
	Bf_depth_try_sum        uint32
	Bf_queue_len            uint32
	Bf_queue_len_sum        uint32
	Bf_when_last_cycle      int64
	Bf_active               uint32
}

type Slurmdb_qos_rec struct {
	Description string
	Id          uint32
	Flags       uint32 /* flags for various things to enforce or
	   override other limits */
	Grace_time   uint32 /* preemption grace time */
	Grp_cpu_mins uint64 /* max number of cpu minutes all jobs
	* running under this qos can run for */
	Grp_cpu_run_mins uint64 /* max number of cpu minutes all jobs
	 * running under this qos can
	 * having running at one time */
	Grp_cpus uint32 /* max number of cpus this qos
	   can allocate at one time */
	Grp_jobs uint32 /* max number of jobs this qos can run
	 * at one time */
	Grp_mem uint32 /* max amount of memory this qos
	   can allocate at one time */
	Grp_nodes uint32 /* max number of nodes this qos
	   can allocate at once */
	Grp_submit_jobs uint32 /* max number of jobs this qos can submit at
	 * one time */
	Grp_wall uint32 /* total time in hours this qos can run for */

	Max_cpu_mins_pj uint64 /* max number of cpu mins a job can
	 * use with this qos */
	Max_cpu_run_mins_pu uint64 /* max number of cpu mins a user can
	 * allocate at a given time when
	 * using this qos (Not yet valid option) */
	Max_cpus_pj uint32 /* max number of cpus a job can
	 * allocate with this qos */
	Max_cpus_pu uint32 /* max number of cpus a user can
	 * allocate with this qos at one time */
	Max_jobs_pu uint32 /* max number of jobs a user can
	 * run with this qos at one time */
	Max_nodes_pj uint32 /* max number of nodes a job can
	* allocate with this qos at one time */
	Max_nodes_pu uint32 /* max number of nodes a user can
	* allocate with this qos at one time */
	Max_submit_jobs_pu uint32 /* max number of jobs a user can
	submit with this qos at once */
	Max_wall_pj uint32 /* longest time this
	 * qos can run a job */

	Name string
	//bitstr_t *preempt_bitstr; /* other qos' this qos can preempt */

	Preempt_list []string /* List preempt_list; list of char *'s only used to add or
	 * change the other qos' this can preempt,
	 * when doing a get use the preempt_bitstr */
	Preempt_mode uint16 /* See PREEMPT_MODE_* in slurm/slurm.h */
	Priority     uint32 /* ranged int needs to be a unint for
	 * heterogeneous systems */
	//assoc_mgr_qos_usage_t *usage; /* For internal use only, DON'T PACK */
	Usage_factor float64 /* double, factor to apply to usage in this qos */
	Usage_thres  float64 /* double, percent of effective usage of an
	   association when breached will deny
	   pending and new jobs */
}

type Qos_info_msg struct {
	Record_count uint32
	Qos_list     []Slurmdb_qos_rec
}

type Slurmdb_job_rec struct {
	Alloc_cpus  uint32
	Alloc_nodes uint32
	Account     string
	Associd     uint32
	Blockid     string
	Cluster     string
	Derived_ec  uint32
	Derived_es  string /* aka "comment" */
	Elapsed     uint32
	Eligible    int64
	End         int64
	Exitcode    uint32
	//void *first_step_ptr;
	Gid       uint32
	Jobid     uint32
	Jobname   string
	Lft       uint32
	Partition string
	Nodes     string
	Priority  uint32
	Qosid     uint32
	Req_cpus  uint32
	Req_mem   uint32
	Requid    uint32
	Resvid    uint32
	Show_full uint32
	Start     int64
	State     uint16
	//slurmdb_stats_t stats;
	//List    steps; /* list of slurmdb_step_rec_t *'s */
	Submit        int64
	Suspended     uint32
	Sys_cpu_sec   uint32
	Sys_cpu_usec  uint32
	Timelimit     uint32
	Tot_cpu_sec   uint32
	Tot_cpu_usec  uint32
	Track_steps   uint16
	Uid           uint32
	User          string
	User_cpu_sec  uint32
	User_cpu_usec uint32
	Wckey         string
	Wckeyid       uint32
}
